from enum import Enum

from garpix_utils.logs.enums.get_enums import ActionType, ActionLevel
from garpix_utils.logs.services.logger_iso import ActionElement


class ActionDefault(Enum):
    # 100-109 | Управление учетными записями пользователей
    user_registration = ActionElement(100, ActionType.user_account, 'Регистрация', ActionLevel.info)
    user_create = ActionElement(101, ActionType.user_account, 'Создание пользователя', ActionLevel.info)
    user_change = ActionElement(102, ActionType.user_account, 'Изменение параметров пользователя', ActionLevel.warning)
    user_delete = ActionElement(103, ActionType.user_account, 'Удаление пользователя', ActionLevel.warning)

    # 110-119 | Идентификация и аутентификация субъекта доступа
    user_login = ActionElement(110, ActionType.authentication, 'Вход в систему', ActionLevel.info)
    user_logout = ActionElement(111, ActionType.authentication, 'Выход из системы', ActionLevel.info)

    # 120-129 | Управление атрибутами доступа
    user_access = ActionElement(120, ActionType.user_access_attribute, 'Изменение привилегий пользователя', ActionLevel.warning)

    # 130-139 | Доступ к защищаемой информации
    user_exist = ActionElement(130, ActionType.access_information, 'Проверка существования пользователя', ActionLevel.info)

    # 140-149 | Удаление информации
    any_entity_delete = ActionElement(140, ActionType.delete_information, 'Удаление сущностей', ActionLevel.info)

    # 150-159 | Добавление информации
    any_entity_create = ActionElement(150, ActionType.create_information, 'Создание сущностей', ActionLevel.info)

    # 160-169 | Изменение информации
    any_entity_change = ActionElement(160, ActionType.change_information, 'Изменение сущностей', ActionLevel.info)

    # 170-179 | Изменение конфигураций
    configuration_change = ActionElement(170, ActionType.change_configuration, 'Изменение конфигураций', ActionLevel.info)

    '''
        100-109 | Управление учетными записями пользователей
        110-119 | Идентификация и аутентификация субъекта доступа
        120-129 | Управление атрибутами доступа
        130-139 | Доступ к защищаемой информации
        140-149 | Удаление информации
        150-159 | Добавление информации
        160-169 | Изменение информации
        170-179 | Изменение конфигураций
    '''



from django.conf import settings
import logging

from garpix_utils.logs.enums.get_enums import ActionType, ActionLevel, ActionResult


class ActionElement:

    def __init__(self, action_id: int, action_type: ActionType, name: str, level: ActionLevel):
        self.id = action_id
        self.name = name
        self.type = action_type
        self.level = level


class LoggerIso:

    def __init__(self, logger_name=settings.ISO_LOGS_NAME):
        self.logger = logging.getLogger(logger_name)

    def create_log(self, action: ActionElement, obj, obj_address, result: ActionResult,
                   params=None, sbj=None, sbj_address=None, msg=''):

        log = f'id={action.id} | act=\"{action.name}\"'
        log += f' | sbj=\"{sbj}\"' if sbj else ''
        log += f' | sbj_addr=\"{sbj_address}\"' \
               f' | act_type=\"{action.type.value}\"' \
               f' | lvl={action.level.value} | obj={obj}' \
               f' | obj_addr=\"{obj_address}\"' \
               f' | result={result.value}'
        log += f' | change=\"{params}\"' if params else ''
        log += f' | msg=\"{msg}\"'
        return log

    def write(self, act: ActionElement, obj, obj_address, result: ActionResult, params=None, sbj=None, sbj_address=None, msg=""):
        log = self.create_log(act, obj, obj_address, result, params, sbj, sbj_address, msg)
        self.logger.info(log)

    def write_string(self, string):
        self.logger.info(string)

    @staticmethod
    def get_client_ip(request):
        # Используется для поулчения адреса субъекта
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip



from garpix_utils.logs.enums.get_enums import Action
from garpix_utils.logs.services.logger_iso import LoggerIso, ActionResult


class CreateLogMixin:
    log_msg_delete = None
    log_msg_create = None
    log_msg_change = None

    @staticmethod
    def log_delete(logger, request, obj, action):
        msg = CreateLogMixin.log_msg_delete if CreateLogMixin.log_msg_delete else f'Объект {obj.__class__.__name__} c id={obj.pk} был удален'
        return logger.create_log(action=action,
                                 obj=obj.__class__.__name__,
                                 obj_address=request.path,
                                 result=ActionResult.success,
                                 sbj=request.user.username,
                                 sbj_address=LoggerIso.get_client_ip(request),
                                 msg=msg)

    @staticmethod
    def log_change_or_create(logger, request, obj, change):
        if change:
            old_obj = obj.__class__.objects.get(pk=obj.pk) if change else None
            changed_fields = ''
            fields = obj._meta.fields
            for field in fields:
                field_name = field.name
                old_value = getattr(old_obj, field_name)
                new_value = getattr(obj, field_name)
                if old_obj and old_value != new_value:
                    changed_fields += f'{field.verbose_name}: {old_value} -> {new_value};'
            params = changed_fields
            action = Action.any_entity_change.value
            msg = CreateLogMixin.log_msg_change if CreateLogMixin.log_msg_change else f'Объект {obj.__class__.__name__} c id={obj.pk} был изменен'
        else:
            action = Action.any_entity_create.value
            title = f' с названием {obj.title}' if hasattr(obj, 'title') else ''
            msg = CreateLogMixin.log_msg_create if CreateLogMixin.log_msg_create else f'Объект {obj.__class__.__name__}{title} был добавлен'
            params = None

        return logger.create_log(action=action,
                                 obj=obj.__class__.__name__,
                                 obj_address=request.path,
                                 result=ActionResult.success,
                                 sbj=request.user.username,
                                 params=params,
                                 sbj_address=LoggerIso.get_client_ip(request),
                                 msg=msg)
